import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleOrdinal, max, format, extent } from 'd3';
import ReactDropdown from 'react-dropdown';

const width = 960;
const menuHeight = 75;
const height = 500 - menuHeight;
const margin = { top: 30, right: 200, bottom: 55, left: 90 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;

const attributes = [
  { value: 'sepal_length', label: 'Sepal Length' },
  { value: 'sepal_width', label: 'Sepal Width' },
  { value: 'petal_length', label: 'Petal Length' },
  { value: 'petal_width', label: 'Petal Width' }
];

const getLabel = value => {
  for(let i = 0; i < attributes.length; i++){
    if(attributes[i].value === value){
      return attributes[i].label;
    }
  }
};

const csvUrl = 'iris.csv';

// load .csv file
const useData = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    const row = d => {
      d.sepal_length = +d['sepal length'];
      d.sepal_width = +d['sepal width'];
      d.petal_length = +d['petal length'];
      d.petal_width = +d['petal width'];
      return d;
    };
    csv(csvUrl, row).then(setData);
    
  }, []);
  return data;
};

const AxisX = ({ 
  xScale, innerHeight, tickFormat, tickOffset = 3 
}) =>
  xScale.ticks().map(tickValue => (
    <g
      className="tick"
      key={tickValue}
      transform={`translate(${xScale(tickValue)},0)`}
    >
      <line y2={innerHeight} />
      <text style={{ textAnchor: 'middle' }} dy=".71em" y={innerHeight + tickOffset}>
        {tickFormat(tickValue)}
      </text>
    </g>
  ));

const AxisY = ({ 
  yScale, innerWidth, tickOffset = 3 
}) =>
  yScale.ticks().map(tickValue => (
    <g className="tick" transform={`translate(0,${yScale(tickValue)})`}>
      <line x2={innerWidth} />
      <text
        key={tickValue}
        style={{ textAnchor: 'end' }}
        x={-tickOffset}
        dy=".32em"
      >
        {tickValue}
      </text>
    </g>
  ));

// menus of choosing attributes of axes
const Dropdown = ({ 
  options, id, selectedValue, onSelectedValueChange
}) => (
  <select id={id} onChange={event => onSelectedValueChange(event.target.value)}>
    {options.map(({ value, label }) => (
      <option value={value} selected={value === selectedValue}>
        {label}
      </option>
    ))}
  </select>
);

// attributes of plots on coordinary system
const Marks = ({
  data, xScale, yScale, colorScale, xValue, yValue, colorValue, tooltipFormat, circleRadius
}) =>
  data.map(d => (
    <circle
      className="mark"
      cx={xScale(xValue(d))}
      cy={yScale(yValue(d))}
      fill={colorScale(colorValue(d))}
      fill-opacity={0.5}
      r={circleRadius}
    >
      <title>{tooltipFormat(xValue(d))}</title>
    </circle>
  ));  

const App = () => {
  const data = useData();

  const initialXAttribute = 'petal_length';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);

  const initialYAttribute = 'petal_width';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);
  
  const colorValue = d => d.class;
  const colorLegendLabel = 'Classes';

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0]);
  
  const colorScale = scaleOrdinal()
  	.domain(data.map(colorValue))
  	.range(['#6b8e23', '#191970', '#ff4500']);
  
  return (
    <>
    	<label for="x-select">x-axis:</label>
     	<Dropdown
      	options={attributes}
      	id="x-select"
       	selectedValue={xAttribute}
       	onSelectedValueChange={setXAttribute}
     	/>
     	<label for="y-select">y-axis:</label>
     	<Dropdown
     	  options={attributes}
     	  id="y-select"
     	  selectedValue={yAttribute}
     	  onSelectedValueChange={setYAttribute}
    	/>
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <AxisX
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${innerHeight /
              2 - 90}) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
          <AxisY yScale={yScale} innerWidth={innerWidth} tickOffset={5} />
          <text
            className="axis-label"
            x={innerWidth / 2 + 430}
            y={innerHeight + xAxisLabelOffset - 45}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>
          <text
            x={innerWidth + 23}
            y={30}
            font-size={'2em'}
            fill={'#635F5D'}
          >
            {'Classes'}
          </text>
          <circle
            cx={innerWidth + 35}
            cy={63}
            r={7}
            fill={'#6b8e23'}
          />
          <text
            x={innerWidth + 55}
            y={70}
            font-size={'1.5em'}
            fill={'#635F5D'}
          >
            {'setosa'}
          </text>
          <circle
            cx={innerWidth + 35}
            cy={103}
            r={7}
            fill={'#191970'}
          />
          <text
            x={innerWidth + 55}
            y={110}
            font-size={'1.5em'}
            fill={'#635F5D'}
          >
            {'versicolor'}
          </text>
          <circle
            cx={innerWidth + 35}
            cy={143}
            r={7}
            fill={'#ff4500'}
          />
          <text
            x={innerWidth + 55}
            y={150}
            font-size={'1.5em'}
            fill={'#635F5D'}
          >
            {'virginica'}
          </text>
          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            colorScale={colorScale}
            xValue={xValue}
            yValue={yValue}
            colorValue={colorValue}
            tooltipFormat={xAxisTickFormat}
            circleRadius={7}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
